@@ -86,6 +86,8 @@ TEST_F(LibraryTest, OptionsChannelInit) {
   opts.lookups = strdup("b");
   optmask |= ARES_OPT_LOOKUPS;
   optmask |= ARES_OPT_ROTATE;
+  opts.resolvconf_path = strdup("/etc/resolv.conf");
+  optmask |= ARES_OPT_RESOLVCONF;
 
   ares_channel channel = nullptr;
   EXPECT_EQ(ARES_SUCCESS, ares_init_options(&channel, &opts, optmask));
@@ -112,6 +114,7 @@ TEST_F(LibraryTest, OptionsChannelInit) {
   EXPECT_EQ(std::string(opts.domains[0]), std::string(opts2.domains[0]));
   EXPECT_EQ(std::string(opts.domains[1]), std::string(opts2.domains[1]));
   EXPECT_EQ(std::string(opts.lookups), std::string(opts2.lookups));
+  EXPECT_EQ(std::string(opts.resolvconf_path), std::string(opts2.resolvconf_path));
 
   ares_destroy_options(&opts);
   ares_destroy_options(&opts2);
@@ -169,6 +172,8 @@ TEST_F(LibraryTest, OptionsChannelAllocFail) {
   opts.lookups = strdup("b");
   optmask |= ARES_OPT_LOOKUPS;
   optmask |= ARES_OPT_ROTATE;
+  opts.resolvconf_path = strdup("/etc/resolv.conf");
+  optmask |= ARES_OPT_RESOLVCONF;
 
   ares_channel channel = nullptr;
   for (int ii = 1; ii <= 8; ii++) {
@@ -354,6 +359,26 @@ CONTAINED_TEST_F(LibraryTest, ContainerChannelInit,
   return HasFailure();
 }
 
+CONTAINED_TEST_F(LibraryTest, ContainerSortlistOptionInit,
+                 "myhostname", "mydomainname.org", filelist) {
+  ares_channel channel = nullptr;
+  struct ares_options opts = {0};
+  int optmask = 0;
+  optmask |= ARES_OPT_SORTLIST;
+  opts.nsort = 0;
+  // Explicitly specifying an empty sortlist in the options should override the
+  // environment.
+  EXPECT_EQ(ARES_SUCCESS, ares_init_options(&channel, &opts, optmask));
+  ares_save_options(channel, &opts, &optmask);
+  EXPECT_EQ(0, opts.nsort);
+  EXPECT_EQ(nullptr, opts.sortlist);
+  EXPECT_EQ(ARES_OPT_SORTLIST, (optmask & ARES_OPT_SORTLIST));
+  ares_destroy_options(&opts);
+
+  ares_destroy(channel);
+  return HasFailure();
+}
+
 NameContentList fullresolv = {
   {"/etc/resolv.conf", " nameserver   1.2.3.4 \n"
                        "search   first.com second.com\n"
@@ -376,6 +401,35 @@ CONTAINED_TEST_F(LibraryTest, ContainerFullResolvInit,
   return HasFailure();
 }
 
+// Allow path for resolv.conf to be configurable
+NameContentList myresolvconf = {
+  {"/tmp/myresolv.cnf", " nameserver   1.2.3.4 \n"
+                       "search   first.com second.com\n"
+                       "lookup bind\n"
+                       "options debug ndots:5\n"
+                       "sortlist 1.2.3.4/16 2.3.4.5\n"}};
+CONTAINED_TEST_F(LibraryTest, ContainerMyResolvConfInit,
+                 "myhostname", "mydomain.org", myresolvconf) {
+  char filename[] = "/tmp/myresolv.cnf";
+  ares_channel channel = nullptr;
+  struct ares_options options = {0};
+  options.resolvconf_path = strdup(filename);
+  int optmask = ARES_OPT_RESOLVCONF;
+  EXPECT_EQ(ARES_SUCCESS, ares_init_options(&channel, &options, optmask));
+
+  optmask = 0;
+  free(options.resolvconf_path);
+  options.resolvconf_path = NULL;
+
+  EXPECT_EQ(ARES_SUCCESS, ares_save_options(channel, &options, &optmask));
+  EXPECT_EQ(ARES_OPT_RESOLVCONF, (optmask & ARES_OPT_RESOLVCONF));
+  EXPECT_EQ(std::string(filename), std::string(options.resolvconf_path));
+
+  ares_destroy_options(&options);
+  ares_destroy(channel);
+  return HasFailure();
+}
+
 NameContentList hostconf = {
   {"/etc/resolv.conf", "nameserver 1.2.3.4\n"
                        "sortlist1.2.3.4\n"  // malformed line
@@ -431,8 +485,8 @@ CONTAINED_TEST_F(LibraryTest, ContainerResolvConfNotReadable,
                  "myhostname", "mydomainname.org", filelist) {
   ares_channel channel = nullptr;
   MakeUnreadable hide("/etc/resolv.conf");
-  // Unavailable /etc/resolv.conf fails initialization.
-  EXPECT_EQ(ARES_EFILE, ares_init(&channel));
+  // Unavailable /etc/resolv.conf falls back to defaults
+  EXPECT_EQ(ARES_SUCCESS, ares_init(&channel));
   return HasFailure();
 }
 CONTAINED_TEST_F(LibraryTest, ContainerNsswitchConfNotReadable,
@@ -470,6 +524,71 @@ CONTAINED_TEST_F(LibraryTest, ContainerSvcConfNotReadable,
   return HasFailure();
 }
 
+NameContentList rotateenv = {
+  {"/etc/resolv.conf", "nameserver 1.2.3.4\n"
+                       "search first.com second.com\n"
+                       "options rotate\n"}};
+CONTAINED_TEST_F(LibraryTest, ContainerRotateInit,
+                 "myhostname", "mydomainname.org", rotateenv) {
+  ares_channel channel = nullptr;
+  EXPECT_EQ(ARES_SUCCESS, ares_init(&channel));
+
+  struct ares_options opts;
+  int optmask = 0;
+  ares_save_options(channel, &opts, &optmask);
+  EXPECT_EQ(ARES_OPT_ROTATE, (optmask & ARES_OPT_ROTATE));
+  ares_destroy_options(&opts);
+
+  ares_destroy(channel);
+  return HasFailure();
+}
+
+CONTAINED_TEST_F(LibraryTest, ContainerRotateOverride,
+                 "myhostname", "mydomainname.org", rotateenv) {
+  ares_channel channel = nullptr;
+  struct ares_options opts = {0};
+  int optmask = ARES_OPT_NOROTATE;
+  EXPECT_EQ(ARES_SUCCESS, ares_init_options(&channel, &opts, optmask));
+
+  optmask = 0;
+  ares_save_options(channel, &opts, &optmask);
+  EXPECT_EQ(ARES_OPT_NOROTATE, (optmask & ARES_OPT_NOROTATE));
+  ares_destroy_options(&opts);
+
+  ares_destroy(channel);
+  return HasFailure();
+}
+
+// Test that blacklisted IPv6 resolves are ignored.  They're filtered from any
+// source, so resolv.conf is as good as any.
+NameContentList blacklistedIpv6 = {
+  {"/etc/resolv.conf", " nameserver 254.192.1.1\n" // 0xfe.0xc0.0x01.0x01
+                       " nameserver fec0::dead\n"  // Blacklisted
+                       " nameserver ffc0::c001\n"  // Not blacklisted
+                       " domain first.com\n"},
+  {"/etc/nsswitch.conf", "hosts: files\n"}};
+CONTAINED_TEST_F(LibraryTest, ContainerBlacklistedIpv6,
+                 "myhostname", "mydomainname.org", blacklistedIpv6) {
+  ares_channel channel = nullptr;
+  EXPECT_EQ(ARES_SUCCESS, ares_init(&channel));
+  std::vector<std::string> actual = GetNameServers(channel);
+  std::vector<std::string> expected = {
+    "254.192.1.1",
+    "ffc0:0000:0000:0000:0000:0000:0000:c001"
+  };
+  EXPECT_EQ(expected, actual);
+
+  struct ares_options opts;
+  int optmask = 0;
+  ares_save_options(channel, &opts, &optmask);
+  EXPECT_EQ(1, opts.ndomains);
+  EXPECT_EQ(std::string("first.com"), std::string(opts.domains[0]));
+  ares_destroy_options(&opts);
+
+  ares_destroy(channel);
+  return HasFailure();
+}
+
 NameContentList multiresolv = {
   {"/etc/resolv.conf", " nameserver 1::2 ;  ;;\n"
                        " domain first.com\n"},
