@@ -647,17 +647,17 @@ TEST_P(MockChannelTest, UnspecifiedFamilyV6) {
 }
 
 TEST_P(MockChannelTest, UnspecifiedFamilyV4) {
+  DNSPacket rsp6;
+  rsp6.set_response().set_aa()
+    .add_question(new DNSQuestion("example.com", ns_t_aaaa));
+  ON_CALL(server_, OnRequest("example.com", ns_t_aaaa))
+    .WillByDefault(SetReply(&server_, &rsp6));
   DNSPacket rsp4;
   rsp4.set_response().set_aa()
     .add_question(new DNSQuestion("example.com", ns_t_a))
     .add_answer(new DNSARR("example.com", 100, {2, 3, 4, 5}));
   ON_CALL(server_, OnRequest("example.com", ns_t_a))
     .WillByDefault(SetReply(&server_, &rsp4));
-  DNSPacket rsp6;
-  rsp6.set_response().set_aa()
-    .add_question(new DNSQuestion("example.com", ns_t_aaaa));
-  ON_CALL(server_, OnRequest("example.com", ns_t_aaaa))
-    .WillByDefault(SetReply(&server_, &rsp6));
 
   HostResult result;
   ares_gethostbyname(channel_, "example.com.", AF_UNSPEC, HostCallback, &result);
@@ -669,17 +669,17 @@ TEST_P(MockChannelTest, UnspecifiedFamilyV4) {
 }
 
 TEST_P(MockChannelTest, UnspecifiedFamilyNoData) {
-  DNSPacket rsp4;
-  rsp4.set_response().set_aa()
-    .add_question(new DNSQuestion("example.com", ns_t_a));
-  ON_CALL(server_, OnRequest("example.com", ns_t_a))
-    .WillByDefault(SetReply(&server_, &rsp4));
   DNSPacket rsp6;
   rsp6.set_response().set_aa()
     .add_question(new DNSQuestion("example.com", ns_t_aaaa))
     .add_answer(new DNSCnameRR("example.com", 100, "elsewhere.com"));
   ON_CALL(server_, OnRequest("example.com", ns_t_aaaa))
     .WillByDefault(SetReply(&server_, &rsp6));
+  DNSPacket rsp4;
+  rsp4.set_response().set_aa()
+    .add_question(new DNSQuestion("example.com", ns_t_a));
+  ON_CALL(server_, OnRequest("example.com", ns_t_a))
+    .WillByDefault(SetReply(&server_, &rsp4));
 
   HostResult result;
   ares_gethostbyname(channel_, "example.com.", AF_UNSPEC, HostCallback, &result);
@@ -690,6 +690,29 @@ TEST_P(MockChannelTest, UnspecifiedFamilyNoData) {
   EXPECT_EQ("{'' aliases=[] addrs=[]}", ss.str());
 }
 
+TEST_P(MockChannelTest, UnspecifiedFamilyCname6A4) {
+  DNSPacket rsp6;
+  rsp6.set_response().set_aa()
+    .add_question(new DNSQuestion("example.com", ns_t_aaaa))
+    .add_answer(new DNSCnameRR("example.com", 100, "elsewhere.com"));
+  ON_CALL(server_, OnRequest("example.com", ns_t_aaaa))
+    .WillByDefault(SetReply(&server_, &rsp6));
+  DNSPacket rsp4;
+  rsp4.set_response().set_aa()
+    .add_question(new DNSQuestion("example.com", ns_t_a))
+    .add_answer(new DNSARR("example.com", 100, {1, 2, 3, 4}));
+  ON_CALL(server_, OnRequest("example.com", ns_t_a))
+    .WillByDefault(SetReply(&server_, &rsp4));
+
+  HostResult result;
+  ares_gethostbyname(channel_, "example.com.", AF_UNSPEC, HostCallback, &result);
+  Process();
+  EXPECT_TRUE(result.done_);
+  std::stringstream ss;
+  ss << result.host_;
+  EXPECT_EQ("{'example.com' aliases=[] addrs=[1.2.3.4]}", ss.str());
+}
+
 TEST_P(MockChannelTest, ExplicitIP) {
   HostResult result;
   ares_gethostbyname(channel_, "1.2.3.4", AF_INET, HostCallback, &result);
@@ -860,6 +883,18 @@ TEST_P(MockChannelTest, CancelImmediate) {
   EXPECT_EQ(0, result.timeouts_);
 }
 
+TEST_P(MockChannelTest, CancelImmediateGetHostByAddr) {
+  HostResult result;
+  struct in_addr addr;
+  addr.s_addr = htonl(0x08080808);
+  
+  ares_gethostbyaddr(channel_, &addr, sizeof(addr), AF_INET, HostCallback, &result);
+  ares_cancel(channel_);
+  EXPECT_TRUE(result.done_);
+  EXPECT_EQ(ARES_ECANCELLED, result.status_);
+  EXPECT_EQ(0, result.timeouts_);
+}
+
 // Relies on retries so is UDP-only
 TEST_P(MockUDPChannelTest, CancelLater) {
   std::vector<byte> nothing;
@@ -990,7 +1025,7 @@ class MockMultiServerChannelTest
     public ::testing::WithParamInterface< std::pair<int, bool> > {
  public:
   MockMultiServerChannelTest(bool rotate)
-    : MockChannelOptsTest(3, GetParam().first, GetParam().second, nullptr, rotate ? ARES_OPT_ROTATE : 0) {}
+    : MockChannelOptsTest(3, GetParam().first, GetParam().second, nullptr, rotate ? ARES_OPT_ROTATE : ARES_OPT_NOROTATE) {}
   void CheckExample() {
     HostResult result;
     ares_gethostbyname(channel_, "www.example.com.", AF_INET, HostCallback, &result);
@@ -1014,6 +1049,12 @@ class NoRotateMultiMockTest : public MockMultiServerChannelTest {
 
 
 TEST_P(RotateMultiMockTest, ThirdServer) {
+  struct ares_options opts = {0};
+  int optmask = 0;
+  EXPECT_EQ(ARES_SUCCESS, ares_save_options(channel_, &opts, &optmask));
+  EXPECT_EQ(0, (optmask & ARES_OPT_NOROTATE));
+  ares_destroy_options(&opts);
+
   DNSPacket servfailrsp;
   servfailrsp.set_response().set_aa().set_rcode(ns_r_servfail)
     .add_question(new DNSQuestion("www.example.com", ns_t_a));
@@ -1053,6 +1094,12 @@ TEST_P(RotateMultiMockTest, ThirdServer) {
 }
 
 TEST_P(NoRotateMultiMockTest, ThirdServer) {
+  struct ares_options opts = {0};
+  int optmask = 0;
+  EXPECT_EQ(ARES_SUCCESS, ares_save_options(channel_, &opts, &optmask));
+  EXPECT_EQ(ARES_OPT_NOROTATE, (optmask & ARES_OPT_NOROTATE));
+  ares_destroy_options(&opts);
+
   DNSPacket servfailrsp;
   servfailrsp.set_response().set_aa().set_rcode(ns_r_servfail)
     .add_question(new DNSQuestion("www.example.com", ns_t_a));
@@ -1091,48 +1138,21 @@ TEST_P(NoRotateMultiMockTest, ThirdServer) {
   CheckExample();
 }
 
+INSTANTIATE_TEST_CASE_P(AddressFamilies, MockChannelTest, ::testing::ValuesIn(ares::test::families_modes));
 
-INSTANTIATE_TEST_CASE_P(AddressFamilies, MockChannelTest,
-                        ::testing::Values(std::make_pair<int, bool>(AF_INET, false),
-                                          std::make_pair<int, bool>(AF_INET, true),
-                                          std::make_pair<int, bool>(AF_INET6, false),
-                                          std::make_pair<int, bool>(AF_INET6, true)));
+INSTANTIATE_TEST_CASE_P(AddressFamilies, MockUDPChannelTest, ::testing::ValuesIn(ares::test::families));
 
-INSTANTIATE_TEST_CASE_P(AddressFamilies, MockUDPChannelTest,
-                        ::testing::Values(AF_INET, AF_INET6));
+INSTANTIATE_TEST_CASE_P(AddressFamilies, MockTCPChannelTest, ::testing::ValuesIn(ares::test::families));
 
-INSTANTIATE_TEST_CASE_P(AddressFamilies, MockTCPChannelTest,
-                        ::testing::Values(AF_INET, AF_INET6));
+INSTANTIATE_TEST_CASE_P(AddressFamilies, MockExtraOptsTest, ::testing::ValuesIn(ares::test::families_modes));
 
-INSTANTIATE_TEST_CASE_P(AddressFamilies, MockExtraOptsTest,
-                        ::testing::Values(std::make_pair<int, bool>(AF_INET, false),
-                                          std::make_pair<int, bool>(AF_INET, true),
-                                          std::make_pair<int, bool>(AF_INET6, false),
-                                          std::make_pair<int, bool>(AF_INET6, true)));
+INSTANTIATE_TEST_CASE_P(AddressFamilies, MockNoCheckRespChannelTest, ::testing::ValuesIn(ares::test::families_modes));
 
-INSTANTIATE_TEST_CASE_P(AddressFamilies, MockNoCheckRespChannelTest,
-                        ::testing::Values(std::make_pair<int, bool>(AF_INET, false),
-                                          std::make_pair<int, bool>(AF_INET, true),
-                                          std::make_pair<int, bool>(AF_INET6, false),
-                                          std::make_pair<int, bool>(AF_INET6, true)));
+INSTANTIATE_TEST_CASE_P(AddressFamilies, MockEDNSChannelTest, ::testing::ValuesIn(ares::test::families_modes));
 
-INSTANTIATE_TEST_CASE_P(AddressFamilies, MockEDNSChannelTest,
-                        ::testing::Values(std::make_pair<int, bool>(AF_INET, false),
-                                          std::make_pair<int, bool>(AF_INET, true),
-                                          std::make_pair<int, bool>(AF_INET6, false),
-                                          std::make_pair<int, bool>(AF_INET6, true)));
+INSTANTIATE_TEST_CASE_P(TransportModes, RotateMultiMockTest, ::testing::ValuesIn(ares::test::families_modes));
 
-INSTANTIATE_TEST_CASE_P(TransportModes, RotateMultiMockTest,
-                        ::testing::Values(std::make_pair<int, bool>(AF_INET, false),
-                                          std::make_pair<int, bool>(AF_INET, true),
-                                          std::make_pair<int, bool>(AF_INET6, false),
-                                          std::make_pair<int, bool>(AF_INET6, true)));
-
-INSTANTIATE_TEST_CASE_P(TransportModes, NoRotateMultiMockTest,
-                        ::testing::Values(std::make_pair<int, bool>(AF_INET, false),
-                                          std::make_pair<int, bool>(AF_INET, true),
-                                          std::make_pair<int, bool>(AF_INET6, false),
-                                          std::make_pair<int, bool>(AF_INET6, true)));
+INSTANTIATE_TEST_CASE_P(TransportModes, NoRotateMultiMockTest, ::testing::ValuesIn(ares::test::families_modes));
 
 }  // namespace test
 }  // namespace ares
